#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/utsname.h>
#include <signal.h>
#include <sys/ioctl.h>


#define PIPE_DATA "woo"
#define PIPE_DATA_LEN 4

int zmd_get_socket_uid (int fd);
char *zmd_get_arch (void);

typedef void (* SignalHandler) (int signum);

static int signal_read_pipe;
static int signal_write_pipe;
static int last_signal;

int
zmd_get_socket_uid (int fd)
{
	struct ucred cred;
	socklen_t size;
	int rc;

	size = sizeof (cred);
	rc = getsockopt (fd, SOL_SOCKET, SO_PEERCRED, &cred, &size);
	if (rc < 0 || size != sizeof (cred)) {
		return -1;
	} else {
		return cred.uid;
	}
}

char *
zmd_get_arch (void)
{
	struct utsname uts;

	if (uname (&uts) == 0) {
		return strdup (uts.machine);
	} else {
		return NULL;
	}
}

static void
signal_reader (SignalHandler handler)
{
	for (;;) {
		fd_set fds;

		FD_ZERO (&fds);
		FD_SET (signal_read_pipe, &fds);

		if (select (signal_read_pipe+1, &fds, NULL, NULL, NULL) > 0) {
			int buf[PIPE_DATA_LEN];
			read (signal_read_pipe, buf, PIPE_DATA_LEN);

			handler (last_signal);
		}
	}
}

static void
signal_handler (int signum)
{
	last_signal = signum;
	write (signal_write_pipe, PIPE_DATA, PIPE_DATA_LEN);
}

void
zmd_signal_handler_start (SignalHandler *handler)
{
	pthread_t thread;
	int pipes[2];

	pipe (pipes);
	signal_read_pipe = pipes[0];
	signal_write_pipe = pipes[1];
	pthread_create (&thread, NULL, signal_reader, handler);
}

void
zmd_signal_handle (int sig)
{
	signal (sig, signal_handler);
}

void
zmd_signal_unhandle (int sig)
{
	signal (sig, SIG_DFL);
}

void
get_terminal_size (int *x, int *y)
{
	struct winsize ws;
	int i;
	FILE *fds[] = { stdin, stdout, stderr };

	for (i = 0; i < 3; i++) {
		if (ioctl (fileno (fds[i]), TIOCGWINSZ, &ws) >= 0) {
			*x = ws.ws_col;
			*y = ws.ws_row;
			break;
		}
	}
}
